/**
 * Required atomic primitives are defined in this file.
 * 
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef SWISSTM_ATOMIC_H_
#define SWISSTM_ATOMIC_H_

#include <atomic_ops.h>

#define aword AO_t

#define atomic_cas_full(addr, old_val, new_val) (AO_compare_and_swap_full((volatile AO_t *)(addr), (AO_t)(old_val), (AO_t)(new_val)))
#define atomic_cas_acquire(addr, old_val, new_val) (AO_compare_and_swap_acquire((volatile AO_t *)(addr), (AO_t)(old_val), (AO_t)(new_val)))
#define atomic_cas_release(addr, old_val, new_val) (AO_compare_and_swap_release((volatile AO_t *)(addr), (AO_t)(old_val), (AO_t)(new_val)))
#define atomic_cas_no_barrier(addr, old_val, new_val) (AO_compare_and_swap((volatile AO_t *)(addr), (AO_t)(old_val), (AO_t)(new_val)))

#define atomic_store_full(addr, val) (AO_store_full((volatile AO_t *)(addr), (AO_t)(val)))
#define atomic_store_release(addr, val) (AO_store_release((volatile AO_t *)(addr), (AO_t)(val)))
#define atomic_store_no_barrier(addr, val) (AO_store((volatile AO_t *)(addr), (AO_t)(val)))

#define atomic_load_acquire(addr) (AO_load_acquire((volatile AO_t *)(addr)))
#define atomic_load_acquire_read(addr) (AO_load_acquire_read((volatile AO_t *)(addr)))
#define atomic_load_no_barrier(addr) (AO_load((volatile AO_t *)(addr)))

#define fetch_and_inc_no_barrier(addr) (AO_fetch_and_add1((volatile AO_t *)(addr)))
#define fetch_and_inc_full(addr) (AO_fetch_and_add1_full((volatile AO_t *)(addr)))
#define fetch_and_inc_acquire(addr) (AO_fetch_and_add1_acquire((volatile AO_t *)(addr)))
#define fetch_and_inc_release(addr) (AO_fetch_and_add1_release((volatile AO_t *)(addr)))

#define atomic_barrier_full() (AO_nop_full())

namespace swisstm {

	template <typename T, T INITIAL_VALUE>
	class AtomicVariable {
		public:
			AtomicVariable() : value(INITIAL_VALUE)
				{  }

			void setValue(T newval) {
				atomic_store_no_barrier(&value, newval);
			}

			T getValue() {
				return (T)atomic_load_no_barrier(&value);
			}

			bool exchangeValue(T oldval, T newval) {
				return atomic_cas_no_barrier(&value, oldval, newval);
			}

		protected:
			T value;
	};

	class CounterOF : protected AtomicVariable<unsigned, 0> {
		public:
			unsigned getNext() {
				return fetch_and_inc_no_barrier(&value);
			}

			unsigned getMax() {
				return getValue();
			}
	};
}

#endif
