/**
 * Common implementation of shared objects.
 * 
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef SWISSTM_OBJECT_H_
#define SWISSTM_OBJECT_H_

#include "memory.h"

namespace swisstm {

	// interface of an object implementation
	class BaseObject : public SwisstmTxAlloced {
		protected:
			virtual BaseObject *clone() const = 0;

		public:
			virtual BaseObject *cloneFull() const = 0;
	};
}

//////////////////////////////////
// object handle implementation //
//////////////////////////////////

#include "object_mixinv_ir_prio_t.h"



//////////////////////////////////////////////
// choose object handle implementation here //
//////////////////////////////////////////////

namespace swisstm {
	typedef ObjectHandleMixinvIRPrioT ObjectHandleBase;
}

namespace swisstm {
	inline void GlobalInitObjectHandle() {
		// do nothing
	}

	inline void ThreadInitObjectHandle() {
		// do nothing
	}

	inline void freeHandle(ObjectHandleBase *handle) {
		tx_free(handle);
	}
}

//////////////////////////////////////////
// general object handle implementation //
//////////////////////////////////////////

namespace swisstm {

	template <class> class Object;

	template<class T>
	class ObjectHandle : public ObjectHandleBase {
		friend class Object<T>;

		protected:
			ObjectHandle(T *obj) : ObjectHandleBase(obj) {
			}
	};

	template <class T>
	class Object : public BaseObject {
		public:
			Object() : handle(NULL) {
			}

			ObjectHandle<T> *getHandle() {
				if(handle == NULL) {
					handle = new ObjectHandle<T>((T *)this);
				}

				return handle;
			}

			Object<T> *cloneFull() const {
				Object<T> *ret = (Object<T> *)clone();
				ret->handle = handle;
				return ret;
			}

		private:
			ObjectHandle<T> * volatile handle;
	};
}

#endif // SWISSTM_OBJECT_H_
